{% extends "joyous/joyous_base.html" %}
{% load wagtailcore_tags wagtailimages_tags %}

{% block content %}
{% with overrides=page.overrides %}
  <div class="content">
    <div class="event clearfix">
      {% if page.status %}
      <div class="event-status">
        {{ page.status_text }}
      </div>
      {% endif %}
      {% if page.cancellation_title %}
        <h1 class="event-heading">{{ page.cancellation_title }}</h1>
      {% else %}
        <h1 class="event-heading">{{ overrides.title }}</h1>
      {% endif %}
      <div class="event-image">
        {% image overrides.image width-350 %}
      </div>
      {% include "joyous/includes/who_when_where.html" with event=page %}
      <div class="event-cancellation-details">
        {{ page.cancellation_details|richtext }}
      </div> 
    </div>
    {% include "joyous/includes/event_overrides.html" %}
  </div>
{% endwith %}
{% endblock %}
