{% load wagtailcore_tags joyous_tags i18n %}
<div class="event-when">
  {{ event.when }}
</div>
{% if event.postponed_from_when %}
<div class="event-postponed-from">
  {% blocktrans with postponed_from=event.postponed_from_when %}Postponed from {{ postponed_from }}{% endblocktrans %}
</div>
{% endif %}
{% next_on event as event_next_on %}
{% if event_next_on %}
<div class="event-next-on">
  {% blocktrans %}Next on {{ event_next_on }}{% endblocktrans %}
</div>
{% endif %}
{% if event.location %}
<div class="event-location">
  {{ event.location }}
  {% location_gmap event.location %}
</div>
{% endif %}
{% if event.website %}
<div class="event-website">
  <a href="{{ event.website }}" target="_blank">{{ event.website }}</a>
</div>
{% endif %}
